/**************************************************************************//**
 * @file     FlashPrg.c
 * @brief    Flash Programming Functions adapted for New Device Flash
 * @version  V1.0.0
 * @date     10. January 2018
 ******************************************************************************/
/*
 * Copyright (c) 2010-2018 Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
#include "..\FlashOS.H"        // FlashOS Structures

typedef          unsigned long     u32;
typedef volatile unsigned long    vu32;

#define ww(addr, value)     (*((u32 volatile *) (addr)) = value)
#define rw(addr)            (*((u32 volatile *) (addr)))
	
#define FMC_BASE        (0x40000000)

#define pRfFLSCTRL              ((_FLS_CTRL_Reg_File*) FMC_BASE)

#define FLC_CMD_KEY_CODE_MAIN  0xADEB0000
#define FLC_CMD_START          0x00000001
#define FLC_CMD_MASK           0x0000000E
#define FLC_CMD_NWS_MASK       0x00000080
#define FLC_CMD_MODE           0x00000100

u32 FLC_CMD_KEY_USER_DATA[8]={0x2D570000,0x58B40000,0x63F20000,0xDC490000,0x59BD0000,0x37A10000,0x4BF00000,0x71C30000};


#define FLC_CMD_WRITE          0
#define FLC_CMD_SECTOR_ERASE   1
#define FLC_CMD_MASS_ERASE     2
#define FLC_0_WS               0x00000000
#define FLC_1_WS               0x00000020
#define FLC_2_WS               0x00000040
#define FLC_3_WS               0x00000060
#define FLC_4_WS               0x00000080
#define FLC_5_WS               0x000000A0
#define FLC_6_WS               0x000000C0
#define FLC_7_WS               0x000000E0

#define FLC_ISR_CMD_END        0x00000001
#define FLC_ISR_ERASE_END      0x00000002


#define Devsz    64

#define USER_DATA_BASE 	0x0000F000

typedef volatile struct {
  vu32 FL_CMD;                                     /* offset  0x00                        */
  vu32 FL_ISR;                                     /* offset  0x004                            */
  vu32 FL_IER;                                     /* offset  0x008                                                            */
  vu32 FL_AR;                                      /* offset  0x00C                        */
  vu32 FL_DR[4];                                   /* offset  0x010                            */
  vu32 FL_ACM;                                     /* offset  0x020  */
  vu32 FL_HCM;
	vu32 FL_DIV;
	
} _FLS_CTRL_Reg_File;

void  Delay (u32 nCount)
{
  for(; nCount != 0; nCount--);
}


/* 
   Mandatory Flash Programming Functions (Called by FlashOS):
                int Init        (unsigned long adr,   // Initialize Flash
                                 unsigned long clk,
                                 unsigned long fnc);
                int UnInit      (unsigned long fnc);  // De-initialize Flash
                int EraseSector (unsigned long adr);  // Erase Sector Function
                int ProgramPage (unsigned long adr,   // Program Page Function
                                 unsigned long sz,
                                 unsigned char *buf);

   Optional  Flash Programming Functions (Called by FlashOS):
                int BlankCheck  (unsigned long adr,   // Blank Check
                                 unsigned long sz,
                                 unsigned char pat);
                int EraseChip   (void);               // Erase complete Device
      unsigned long Verify      (unsigned long adr,   // Verify Function
                                 unsigned long sz,
                                 unsigned char *buf);

       - BlanckCheck  is necessary if Flash space is not mapped into CPU memory space
       - Verify       is necessary if Flash space is not mapped into CPU memory space
       - if EraseChip is not provided than EraseSector for all sectors is called
*/


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  /* Add your Code */
  return (0);                                  // Finished without Errors
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  /* Add your Code */
  return (0);                                  // Finished without Errors
}


/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseChip (void) {

//	u32 i;
  /* Add your Code */
	pRfFLSCTRL ->FL_DIV = 0x40;//Version 1.1.0 added
	
	pRfFLSCTRL ->FL_AR    = 0x00000000;	
	pRfFLSCTRL ->FL_CMD = FLC_CMD_KEY_CODE_MAIN    |
	                      FLC_CMD_MODE             |
	                      FLC_CMD_START            |
	                      FLC_1_WS                 |
	                      (FLC_CMD_MASS_ERASE<<1);
	
	while(!((pRfFLSCTRL ->FL_ISR) & 0x4));
	
	pRfFLSCTRL ->FL_ISR = 0x4;  	
	return (0);                                  // Finished without Errors
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

int EraseSector (unsigned long adr) {

  /* Add your Code */
	pRfFLSCTRL ->FL_DIV = 0x40;//Version 1.1.0 added
	
	pRfFLSCTRL ->FL_AR    = adr;
	
	pRfFLSCTRL ->FL_CMD =	               FLC_CMD_KEY_CODE_MAIN    |
	                                     FLC_CMD_MODE             |
	                                     FLC_CMD_START            |
	                                     FLC_1_WS                 |
	                                     (FLC_CMD_SECTOR_ERASE<<1);
	
	while(!((pRfFLSCTRL ->FL_ISR) & FLC_ISR_ERASE_END));
	
	pRfFLSCTRL ->FL_ISR = FLC_ISR_ERASE_END;
	
	return (0);                                  // Finished without Errors
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  /* Add your Code */
	pRfFLSCTRL ->FL_DIV = 0x40;//Version 1.1.0 added
	
  sz = (sz + 3) & ~3;                                   // Adjust size for Words
   
	while(sz)
	{
		
//		ww(0x4000000C,adr);
//		ww((0x40000010),*((u32 *) buf));
//		ww((0x40000000),0xA5A50121);

	pRfFLSCTRL ->FL_AR    = adr;
	pRfFLSCTRL ->FL_DR[0] = *((u32 *) buf);
		
	pRfFLSCTRL ->FL_CMD =  FLC_CMD_KEY_CODE_MAIN    |
												 FLC_CMD_MODE             |
												 FLC_CMD_START            |
												 FLC_1_WS                 |
												 (FLC_CMD_WRITE<<1);
	
//		
	while(!((pRfFLSCTRL ->FL_ISR) & FLC_ISR_CMD_END));
//	
	pRfFLSCTRL ->FL_ISR = FLC_ISR_CMD_END;	

	adr +=4;
	buf +=4;
  sz  -=4;			
	}
	
  return (0);                                  // Finished without Errors
}




int EraseNVR () {
	
	
	return 0;
}


int ProgramNVR (unsigned long adr, unsigned long sz, unsigned char *buf) {
return 0;
}
