 /******************************************************************************
  * @file    PT32x007x_ifmc.h
  * @author  ӦÿŶ
  * @version V1.6.0
  * @date    2024/2/28
  * @brief    This file contains all the functions prototypes for the IFMC firmware library.
  *          
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef PT32X007X_IFMC_H
#define PT32X007X_IFMC_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "PT32x007x.h"

/**  @brief  IFMC Status
  * @{
  */
typedef enum
{
	IFMC_WriteEnd = 1,
	IFMC_PageEraseEnd,
	IFMC_ChipEraseEnd,
	IFMC_Busy,
	IFMC_CommandError,
	IFMC_KeyError,
	IFMC_AddrError,
	IFMC_Complete,
	IFMC_TimeOut
} IFMC_Status;

#define IFMC_Key_MainCode       ((u32)0xADEB0000) 
#define IFMC_Key_Config         ((u32)0xC5AE0000) 

/** @brief  IFMC Operation Commands **/
#define IFMC_WriteOperation		IFMC_CR_MODE
#define IFMC_OperationStarts	IFMC_CR_STAR

#define IFMC_CMD_Programming   (IFMC_CR_CMD&0x00)
#define IFMC_CMD_PageErase   (IFMC_CR_CMD&0x02)
#define IFMC_CMD_ChipErase   (IFMC_CR_CMD&0x04)

#if defined PT32x007x6x
	#define MAINCODE_END_ADDR    ((u32)0x00007FFF) 
#elif defined PT32x007x8x
  #define MAINCODE_END_ADDR    ((u32)0x0000FFFF) 
#else
  #error "Please select PT32x007x6x or PT32x007x8x used in your application (in PT32x007x_ifmc.h file)"
#endif

#define FLASH_Configuration_ADDR	((u32)0x18000000)
#define Config_END_ADDR    ((u32)0x180001ff) 

#define IS_IFMC_PageAddress(SEL)   ((SEL) < MAINCODE_END_ADDR)
/** @addtogroup IFMC
  * @{
  */
#define IFMC_Latency_0                (IFMC_CR_WAIT&0x00) 		  /*!< IFMC Zero Latency cycle */
#define IFMC_Latency_1                (IFMC_CR_WAIT&0x20)       /*!< IFMC One Latency cycle */
#define IFMC_Latency_2                (IFMC_CR_WAIT&0x40)       /*!< IFMC Two Latency cycle */
#define IS_IFMC_Latency(SEL)	(((SEL) == IFMC_Latency_0) || \
							((SEL) == IFMC_Latency_1) || \
							((SEL) == IFMC_Latency_2))
		
#define IS_InstructionFetchWaitCycle(SEL) 	((SEL) <= 2) 

#define IS_IFMC_Prescaler(SEL) (((0<SEL)&&(SEL<256)))

/** @defgroup IFMC_Timeout_definition  **/
#define IFMC_ER_PRG_TIMEOUT      ((u32)0x000B0000)

/** @defgroup IFMC_FLAG  **/ 
#define IFMC_FLAG_WCI               IFMC_SR_WC
#define IFMC_FLAG_PEC               IFMC_SR_PEC
#define IFMC_FLAG_CEC               IFMC_SR_CEC
#define IFMC_FLAG_BSY               IFMC_SR_BSY
#define IFMC_FLAG_CERR              IFMC_SR_CERR
#define IFMC_FLAG_KERR              IFMC_SR_KERR
#define IFMC_FLAG_AERR              IFMC_SR_AERR
#define IFMC_FLAG_IAPF              IFMC_BSR_IAPF
#define IFMC_FLAG_PSR               (0x100|IFMC_RPSR_PSR)
#define IS_IFMC_FLAG(SEL)      (((SEL) == IFMC_FLAG_WCI)    || \
                                ((SEL) == IFMC_FLAG_PEC)    || \
								                ((SEL) == IFMC_FLAG_CEC)    || \
								                ((SEL) == IFMC_FLAG_BSY)    || \
                                ((SEL) == IFMC_FLAG_CERR)   || \
																((SEL) == IFMC_FLAG_KERR)   || \
								                ((SEL) == IFMC_FLAG_AERR)   || \
																((SEL) == IFMC_FLAG_IAPF)   || \
																((SEL) == IFMC_FLAG_PSR))
                                                                                           
#define IFMC_IT_WCI             IFMC_IE_WCIE
#define IFMC_IT_PECI            IFMC_IE_PECIE
#define IFMC_IT_CECI            IFMC_IE_CECIE
#define IFMC_IT_CERI            IFMC_IE_CERIE
#define IFMC_IT_KERI            IFMC_IE_KERIE
#define IFMC_IT_AERI            IFMC_IE_AERIE                              
#define IS_IFMC_IT(SEL)        (((SEL) == IFMC_IT_WCI)  || \
                                ((SEL) == IFMC_IT_PECI) || \
								                ((SEL) == IFMC_IT_CECI) || \
								                ((SEL) == IFMC_IT_CERI) || \
								                ((SEL) == IFMC_IT_KERI) || \
																((SEL) == IFMC_IT_AERI))



/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */
void IFMC_SetLatency(u32 IFMC_Latency);
void IFMC_SetPrescaler(u8 Prescaler);
FlagStatus IFMC_WaitIFMCFree(u32 IFMC_FLAG);
FlagStatus IFMC_ProgramWord(u32 Address, u32 Data);
u32 IFMC_ReadWord(u32 Address);
u16 IFMC_ReadHalfWord(u32 Address);
u8 IFMC_ReadByte(u32 Address);
void IFMC_ITConfig(u32 IFMC_IT, FunctionalState NewState);
FlagStatus IFMC_GetFlagStatus(u32 IFMC_FLAG);
void IFMC_ClearFlag(u32 IFMC_FLAG);
FlagStatus IFMC_EraseChip(void);
u32 IFMC_ErasePage(u32 addr);
IFMC_Status IFMC_GetIFMCStatus(void);
IFMC_Status IFMC_WaitForLastOperation(u32 Timeout);
IFMC_Status IFMC_IAPConfig(FunctionalState NewState);
/**
  * @}
  */

  
#ifdef __cplusplus
}
#endif

#endif 
