/******************************************************************************
  * @file    PT32x007x_ifmc.c
  * @author  ӦÿŶ
  * @version V1.6.0
  * @date    2024/2/28
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the IFMC peripheral:
  *           + Initialization and Configuration
  *           + Flash Erase and Read\Write Function
  *           + Interrupts and flags management
  *
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32x007x_ifmc.h"


/** @defgroup IFMC
  * @brief IFMC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define WriteWord_CR_MASK   (u32)0xFFFFFEF8
#define IFMC_TimeOut_Time  ((u32)0x00040000)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/**
  * @brief  Sets the code latency value.
  * @param  IFMC_Latency: specifies the IFMC Latency value.
  *          This parameter can be one of the following values:
  *             @arg IFMC_Latency_0: IFMC Zero Latency cycle
  *             @arg IFMC_Latency_1: IFMC One Latency cycle
	*             @arg IFMC_Latency_2: IFMC Two Latency cycle
  * @retval None
  */
void IFMC_SetLatency(u32 IFMC_Latency)
{
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_IFMC_Latency(IFMC_Latency));
	/* Read the ACR register */
	tmpreg = IFMC->CR;
	/* Sets the Latency value */
	tmpreg &= ~(IFMC_CR_WAIT);
	tmpreg |= IFMC_Latency|IFMC_Key_MainCode;
	/* Write the ACR register */
	IFMC->CR = tmpreg;
}


/**
  * @brief  Sets the IFMC Prescaler.
  * @param  Prescaler: specifies the IFMC PSC value.
  *		    This parameter must be less than 64
  * @retval None
  */
void IFMC_SetPrescaler(u8 Prescaler)
{
	assert_param(IS_IFMC_Prescaler(Prescaler));
	/* Write the PSC register */
	IFMC->PSC = Prescaler;
}

FlagStatus IFMC_WaitIFMCFree(u32 IFMC_FLAG)
{
	u32 Timeout = IFMC_TimeOut_Time;
	FlagStatus bitstatus = RESET;
	while(IFMC_GetFlagStatus(IFMC_FLAG) && (Timeout != 0x00))
	{
		Timeout--;
	}
	if(Timeout)
	{
		bitstatus = SET;
	}
	else
	{
		bitstatus = RESET;
	}
	return bitstatus;
}

FlagStatus IFMC_ProgramWord(u32 Address, u32 Data)
{
	FlagStatus bitstatus = RESET;
	u32 tmpreg = 0;

	bitstatus=IFMC_WaitIFMCFree(IFMC_FLAG_BSY);
	if(bitstatus==SET)
	{
		IFMC->AR = Address;
		IFMC->DR  = Data;
		
		tmpreg = IFMC->CR;
		tmpreg &= WriteWord_CR_MASK;
		
		if(((Address) < MAINCODE_END_ADDR) && ((Address & 0x03) == 0))
		{
			tmpreg |= (IFMC_Key_MainCode | IFMC_CR_STAR | IFMC_CMD_Programming | IFMC_CR_MODE);
			IFMC->CR = tmpreg;
		}
		else
		{
			bitstatus=RESET;
		}
		
	}
	return bitstatus;
}

/**
  * @brief  Erase full flash chip.
  * @note
  * @note
  *
  * @param
  * @note
  *
  * @retval
  *
  */
FlagStatus IFMC_EraseChip(void)
{
	FlagStatus bitstatus = RESET;
	u32 tmpreg = 0;
	bitstatus=IFMC_WaitIFMCFree(IFMC_FLAG_BSY);
	if(bitstatus==SET)
	{
		tmpreg = IFMC->CR;
		tmpreg &= WriteWord_CR_MASK;
		tmpreg |= (IFMC_Key_MainCode | IFMC_CR_STAR | IFMC_CMD_ChipErase | IFMC_CR_MODE);
		IFMC->CR = tmpreg;
	}
	return bitstatus;
}

/**
  * @brief  Erase full flash chip.
  * @note
  * @note
  *
  * @param
  * @note
  *
  * @retval
  *
  */
u32 IFMC_ErasePage(u32 addr)
{
	FlagStatus bitstatus = RESET;
	u32 tmpreg = 0;
	/* Check the parameters */
	assert_param(IS_IFMC_PageAddress(addr));
	bitstatus=IFMC_WaitIFMCFree(IFMC_FLAG_BSY);
	if(bitstatus==SET)
	{
		IFMC->AR = addr;
		tmpreg = IFMC->CR;
		tmpreg &= WriteWord_CR_MASK;
		tmpreg |= (IFMC_Key_MainCode | IFMC_CR_STAR | IFMC_CMD_PageErase | IFMC_CR_MODE);
		IFMC->CR = tmpreg;
	}
	return bitstatus;
}

/**
  * @brief  Read a word at a specified address.
  * @note
  * @note
  *
  * @param
  * @note
  *
  * @retval
  *
  */
u32 IFMC_ReadWord(u32 Address)
{
	return *(__IO u32*) Address;
}
/**
  * @brief  Read a half word at a specified address.
  * @note
  * @note
  *
  * @param
  * @param
  * @retval
  *
  */
u16 IFMC_ReadHalfWord(u32 Address)
{
	return *(__IO u16*) Address;
}

/**
  * @brief Read a Byte at a specified address.
  * @note
  * @note
  *
  * @param
  * @param
  * @retval
  *
  */
u8 IFMC_ReadByte(u32 Address)
{
	return *(__IO u8*) Address;
}


/**
  */
void IFMC_ITConfig(u32 IFMC_IT, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_IFMC_IT(IFMC_IT));
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	if(NewState != DISABLE)
	{
		/* Enable the interrupt sources */
		IFMC->IE |= IFMC_IT;
	}
	else
	{
		/* Disable the interrupt sources */
		IFMC->IE &= ~IFMC_IT;
	}
}

/**
  * @brief  Checks whether the specified IFMC flag is set or not.
  * @param  IFMC_FLAG: specifies the IFMC flag to check.
  *          This parameter can be one of the following values:
  *             @arg IFMC_FLAG_BSY: IFMC write/erase operations in progress flag
  *             @arg IFMC_FLAG_PGERR: IFMC Programming error flag flag
  *             @arg IFMC_FLAG_WRPERR: IFMC Write protected error flag
  *             @arg IFMC_FLAG_EOP: IFMC End of Programming flag
  * @retval The new state of IFMC_FLAG (SET or RESET).
  */
FlagStatus IFMC_GetFlagStatus(u32 IFMC_FLAG)
{
	FlagStatus bitstatus = RESET;
	assert_param(IS_IFMC_FLAG(IFMC_FLAG));
	if(IFMC_FLAG == IFMC_FLAG_IAPF)
	{
		if((IFMC->BSR & IFMC_FLAG) != (u32)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	else if(IFMC_FLAG == IFMC_FLAG_PSR)
	{
		if((IFMC->RPSR & (IFMC_FLAG & IFMC_RPSR_PSR)) != (u32)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	else
	{
		if((IFMC->SR & IFMC_FLAG) != (u32)RESET)
		{
			bitstatus = SET;
		}
		else
		{
			bitstatus = RESET;
		}
	}
	return bitstatus;
}

/**
  * @brief  Clears the IFMC's pending flags.
  * @param  IFMC_FLAG: specifies the IFMC flags to clear.
  *          This parameter can be any combination of the following values:
  *             @arg IFMC_FLAG_WOV	IFMC дɱ־
  *             @arg IFMC_FLAG_POV	IFMCɱ־
  *             @arg IFMC_FLAG_COV	IFMCⱣʱɱ־
  *             @arg IFMC_FLAG_BUSY    IFMC־
  *             @arg IFMC_FLAG_CERR   IFMC־
  *             @arg IFMC_FLAG_KERR   IFMC־
  *             @arg IFMC_FLAG_AERR    IFMCַ־
  * @retval None
  */
void IFMC_ClearFlag(u32 IFMC_FLAG)
{
	/* Check the parameters */
	assert_param(IS_IFMC_FLAG(IFMC_FLAG));
	IFMC->SR |= IFMC_FLAG;
}

/**
  * @brief  Returns the IFMC Status.
  * @param  None
  * @retval IFMC Status: The returned value can be:
	*         @arg IFMC_Busy: Operation busy status
	*         @arg IFMC_KeyError: Key error status
	*         @arg IFMC_AddrError: Address error status
	*         @arg IFMC_Complete:  Complete status
  */
IFMC_Status IFMC_GetIFMCStatus(void)
{
    IFMC_Status IFMCstatus = IFMC_Complete;

    if((IFMC->SR & IFMC_SR_BSY) == IFMC_SR_BSY)
    {
        IFMCstatus = IFMC_Busy;
    }
    else
    {
        if((IFMC->SR & IFMC_SR_KERR) == IFMC_SR_KERR)
        {
            IFMCstatus = IFMC_KeyError;
        }
        else
        {
            if((IFMC->SR & IFMC_SR_AERR) == IFMC_SR_AERR)
            {
                IFMCstatus = IFMC_AddrError;
            }
            else
            {
                IFMCstatus = IFMC_Complete;
            }
        }
    }
    /* Return the IFMC Status */
    return IFMCstatus;
}
/**
  * @brief  Waits for a IFMC operation to complete or a TIMEOUT to occur.
  * @param  Timeout: IFMC programming Timeout
  * @retval IFMC_Status: The returned value can be: IFMC_Busy,
  *         IFMC_KeyError, IFMC_AddrError ,IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_WaitForLastOperation(u32 Timeout)
{
    IFMC_Status status = IFMC_Complete;
    /* Check for the IFMC Status */
    status = IFMC_GetIFMCStatus();

    /* Wait for a IFMC operation to complete or a TIMEOUT to occur */
    while((IFMC_GetIFMCStatus() == IFMC_Busy) && (Timeout != 0x00))
    {
        status = IFMC_GetIFMCStatus();
        Timeout--;
    }

    if(Timeout == 0x00)
    {
        status = IFMC_TimeOut;
    }

    /* Return the operation status */
    return status;
}
/**
  * @brief  Enables or disables IAP function of the specified IFMC peripheral.
  * @param  NewState: new state of the IAP funtcion.
  *          This parameter can be: ENABLE or DISABLE.
  * @retval IFMC_Status: The returned value can be: IFMC_Busy,
  *         IFMC_KeyError, IFMC_AddrError, IFMC_Complete or IFMC_TimeOut.
  */
IFMC_Status IFMC_IAPConfig(FunctionalState NewState)
{
    u32 tmpreg = 0;
    u32 tmpid = ID->UDID;
    IFMC_Status status = IFMC_Complete;
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /* Wait for last operation to be completed */
    status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);	//ȴһ
    tmpreg = IFMC->CR;
    tmpreg &= WriteWord_CR_MASK;
    if(status == IFMC_Complete)
    {
        IFMC->AR  = FLASH_Configuration_ADDR;
        /* The configuration area is erased by page */
        tmpreg |= IFMC_Key_Config | IFMC_CR_MODE | IFMC_CMD_PageErase | IFMC_OperationStarts;
        IFMC->CR = tmpreg;
        status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
        if(status == IFMC_Complete)
        {
					IFMC->AR  = FLASH_Configuration_ADDR;
					IFMC->DR  = tmpid;
					/* Data is programming into the configuration area area */
					tmpreg = IFMC->CR;
					tmpreg &= WriteWord_CR_MASK;
					tmpreg |= IFMC_Key_Config | IFMC_WriteOperation | IFMC_CMD_Programming | IFMC_OperationStarts;
					IFMC->CR = tmpreg;
					status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
					if(status != IFMC_Complete)
					{
						return status;
					}
					if(NewState != DISABLE)
					{
						IFMC->AR  = FLASH_Configuration_ADDR + 4;
						IFMC->DR  = 0xFFD303FF;
						/* Data is programming into the configuration area area */
						tmpreg |= IFMC_Key_Config | IFMC_WriteOperation | IFMC_CMD_Programming | IFMC_OperationStarts;
						IFMC->CR = tmpreg;
						status = IFMC_WaitForLastOperation(IFMC_ER_PRG_TIMEOUT);
					}
        }
    }
    /* Return the Erase Status */
    return status;
}
/**
  * @}
  */


